# Copyright (C) 2020, 2021, 2022, 2023 Serghei Iakovlev <egrep@protonmail.ch>
#
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this file.  If not, see <https://www.gnu.org/licenses/>.

"""The top-level module for Gstore.

This module tracks the version of the package as well as the base
package info used by various functions within Gstore.

Modules:

    __main__
    args
    cli
    client
    env
    exceptions
    logger
    models
    repo

Misc variables:

    __copyright__
    __version__
    __license__
    __author__
    __author_email__
    __url__
    __description__

Refer to the `documentation <https://gstore.readthedocs.io/>`_ for
details on the use of this package.

"""

__copyright__ = 'Copyright (C) 2020, 2021, 2022, 2023 Serghei Iakovlev'
__version__ = '0.6.1'
__license__ = 'GPLv3+'
__author__ = 'Serghei Iakovlev'
__author_email__ = 'egrep@protonmail.ch'
__url__ = 'https://gstore.readthedocs.io'
__description__ = 'Synchronize GitHub repositories made easy.'
